package org.jboss.security;

package .tmp;

import java.lang.reflect.Method;
import java.util.HashMap;
import javax.ejb.EJBContext;
import org.jboss.security.SecurityProxy;

public abstract class AbstractSecurityProxy implements SecurityProxy {
  private HashMap methodMap;
  
  private Method setContextMethod;
  
  private Method setBeanMethod;
  
  protected Object delegate;
  
  protected boolean strict = false;
  
  AbstractSecurityProxy(Object delegate) {
    this.delegate = delegate;
    this.methodMap = new HashMap();
  }
  
  public void init(Class beanHome, Class beanRemote, Object securityMgr) throws InstantiationException {
    init(beanHome, beanRemote, null, null, securityMgr);
  }
  
  public void init(Class beanHome, Class beanRemote, Class beanLocalHome, Class beanLocal, Object securityMgr) throws InstantiationException {
    mapHomeMethods(beanHome);
    mapHomeMethods(beanLocalHome);
    mapRemoteMethods(beanRemote);
    mapRemoteMethods(beanLocal);
    try {
      Class[] parameterTypes = { EJBContext.class };
      this.setContextMethod = this.delegate.getClass().getMethod("setEJBContext", parameterTypes);
    } catch (Exception ignore) {}
    try {
      Class[] parameterTypes = { Object.class };
      this.setBeanMethod = this.delegate.getClass().getMethod("setBean", parameterTypes);
    } catch (Exception ignore) {}
    try {
      Class[] parameterTypes = new Class[0];
      Object[] args = new Object[0];
      Method isStrict = this.delegate.getClass().getMethod("isStrict", parameterTypes);
      Boolean flag = (Boolean)isStrict.invoke(this.delegate, args);
      this.strict = flag.booleanValue();
    } catch (Exception ignore) {}
  }
  
  public void setEJBContext(EJBContext ctx) {
    if (this.setContextMethod != null) {
      Object[] args = { ctx };
      try {
        this.setContextMethod.invoke(this.delegate, args);
      } catch (Exception e) {
        e.printStackTrace();
      } 
    } 
  }
  
  public void invokeHome(Method m, Object[] args) throws Exception {
    Method delegateMethod = (Method)this.methodMap.get(m);
    if (delegateMethod != null)
      invokeHomeOnDelegate(delegateMethod, args, this.delegate); 
  }
  
  public void invoke(Method m, Object[] args, Object bean) throws Exception {
    Method delegateMethod = (Method)this.methodMap.get(m);
    if (delegateMethod != null) {
      if (this.setBeanMethod != null) {
        Object[] bargs = { bean };
        try {
          this.setBeanMethod.invoke(this.delegate, bargs);
        } catch (Exception e) {
          e.printStackTrace();
          throw new SecurityException("Failed to set bean on proxy" + e.getMessage());
        } 
      } 
      invokeOnDelegate(delegateMethod, args, this.delegate);
    } 
  }
  
  protected void mapHomeMethods(Class beanHome) {
    if (beanHome == null)
      return; 
    Class delegateClass = this.delegate.getClass();
    Method[] methods = beanHome.getMethods();
    for (int m = 0; m < methods.length; m++) {
      Method hm = methods[m];
      Class[] parameterTypes = hm.getParameterTypes();
      String name = hm.getName();
      name = "ejb" + Character.toUpperCase(name.charAt(0)) + name.substring(1);
      try {
        Method match = delegateClass.getMethod(name, parameterTypes);
        this.methodMap.put(hm, match);
      } catch (NoSuchMethodException e) {
        name = hm.getName();
        try {
          Method match = delegateClass.getMethod(name, parameterTypes);
          this.methodMap.put(hm, match);
        } catch (NoSuchMethodException e2) {
          if (this.strict) {
            String msg = "Missing home method:" + hm + " in delegate";
            throw new SecurityException(msg);
          } 
        } 
      } 
    } 
  }
  
  protected void mapRemoteMethods(Class beanRemote) {
    if (beanRemote == null)
      return; 
    Class delegateClass = this.delegate.getClass();
    Method[] methods = beanRemote.getMethods();
    for (int m = 0; m < methods.length; m++) {
      Method rm = methods[m];
      Class[] parameterTypes = rm.getParameterTypes();
      String name = rm.getName();
      try {
        Method match = delegateClass.getMethod(name, parameterTypes);
        this.methodMap.put(rm, match);
      } catch (NoSuchMethodException e) {
        if (this.strict) {
          String msg = "Missing method:" + rm + " in delegate";
          throw new SecurityException(msg);
        } 
      } 
    } 
  }
  
  protected abstract void invokeHomeOnDelegate(Method paramMethod, Object[] paramArrayOfObject, Object paramObject) throws Exception;
  
  protected abstract void invokeOnDelegate(Method paramMethod, Object[] paramArrayOfObject, Object paramObject) throws Exception;
}
